from setuptools import find_packages, setup

setup(
    name='netbox_ipcalculator',
    version='1.1',
    description='Sentia IP Calculator',
    url='https://github.com/PieterL75/netbox_ipcalculator',
    author='Pieter Lambrecht',
    license='Apache 2.0',
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)
